MAX_PAGE = 10
MAX_ADDRESSES = 30

POOL_INCLUDES = ["base_token", "quote_token", "dex", "network"]
NETWORK_POOL_INCLUDES = ["base_token", "quote_token", "dex"]
TOKEN_INCLUDES = ["top_pools"]
TOKEN_INFO_INCLUDES = ["network"]

TIMEFRAMES = ["day", "hour", "minute"]
DAY_AGGREGATES = [1]
HOUR_AGGREGATES = [1, 4, 12]
MINUTE_AGGREGATES = [1, 5, 15]
OHLCV_LIMIT = 1000

CURRENCIES = ["usd", "token"]
TOKENS = ["base", "quote"]
